/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <string.h>
#include <stdlib.h>

#include <glib.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/gnome-dialog.h>
#include <libgnomeui/gnome-stock.h>

#include "article-db.h"
#include "articlelist.h"
#include "debug.h"
#include "folderlist.h"
#include "globals.h"
#include "group.h"
#include "grouplist.h"
#include "newsrc.h"
#include "nntp.h"
#include "prefs.h"
#include "gui.h"
#include "queue.h"
#include "server.h"
#include "task-bodies.h"
#include "task-headers.h"
#include "task-grouplist.h"
#include "task-func-ptr.h"
#include "util.h"


static void real_group_empty (Group *group);


Group*
group_new (Server * server, const char * name)
{
	Group * g = g_new0 (Group, 1);
	debug1 (DEBUG_PAN_OBJECT, "group_new: %p", g);
        group_constructor (g, group_destructor, server, name);
	return g;
}

void
group_constructor (Group                * g,
                   PanObjectDestructor    dtor,
		   Server               * server,
		   const char           * name)
{
	pan_warn_if_fail (is_nonempty_string(name));

	pan_object_constructor (PAN_OBJECT(g), dtor);

	debug1 (DEBUG_PAN_OBJECT, "group_constructor: %p", g);
	g->server = server;
	g->name = g_strdup (name);
	g->description = NULL;
	g->download_dir = NULL;
	g->flags = 0;
	g->state_filter = ~STATE_FILTER_KILLFILE;
	g->type = TYPE_GROUP;
	g->article_low = (gulong)0;
	g->article_high = (gulong)0;
	g->article_qty = (gint)0;
	g->article_read_qty = (gint)0;
	g->newsrc = newsrc_new (NULL, g->article_low, g->article_high);
	g->sort_style = 0;
}

void
group_destructor (PanObject * o)
{
	Group * g = GROUP(o);

	g_return_if_fail (o!=NULL);

	if (g->dirty){
		g_message ("Saving group [%s]", g->name);
		grouplist_save_group (g);
	}

	debug1 (DEBUG_PAN_OBJECT, "group_destructor: %p", g);
	g_free (g->name);
	g_free (g->description);
	pan_object_unref (PAN_OBJECT(g->newsrc));

	pan_object_destructor (o);
}

/**
***
**/

void
group_set_dirty (Group * group)
{
	g_return_if_fail (group != NULL);
	group->dirty = 1;
}

void
group_set_download_dir (Group * group, const gchar* download_dir)
{
	g_free (group->download_dir);
	group->download_dir = g_strstrip(g_strdup (download_dir));
	group_set_dirty (group);
}

void
group_set_flags (Group * group, guint flags)
{
	group->flags = flags;
	group_set_dirty (group);
}

void
group_set_state_filter (Group * group, guint state_filter)
{
	group->state_filter = state_filter;
	group_set_dirty (group);
}

void
group_set_sort_style (Group * group, int sort_style)
{
	group->sort_style = sort_style;
	group_set_dirty (group);
}

void
group_set_article_range (Group * group,
                         gulong article_low,
			 gulong article_high)
{
	group->article_low = article_low;
	group->article_high = article_high;
	group_set_dirty (group);
}

void
group_set_article_qty (Group * group, gint article_qty)
{
	group->article_qty = article_qty;
	group_set_dirty (group);
	grouplist_update_row (group);
}
void
group_inc_article_qty (Group * group, gint inc)
{
	group_set_article_qty (group, group->article_qty + inc);
}

void
group_set_article_read_qty (Group * group, gint article_read_qty)
{
	article_read_qty = MAX (article_read_qty, 0);
	group->article_read_qty = article_read_qty;
	group_set_dirty (group);
	grouplist_update_row (group);
}
void
group_inc_article_read_qty (Group * group, gint inc)
{
	group_set_article_read_qty (group, group->article_read_qty + inc);
}

void
group_mark_all_read (Group * group, gboolean read)
{
	g_return_if_fail (group != NULL);
	newsrc_mark_all (group->newsrc, read);
	group_set_article_read_qty (group, (read ? group->article_qty : 0));
}


int
compare_group (const Group *a, const Group *b)
{
	return strcmp(a->name, b->name);
}

int
compare_group_std (const void* va, const void* vb)
{
	return compare_group (*(const Group**)va, *(const Group**)vb);
}


/* empty the group, delete the group database file and gnome_config sections */
static void
real_group_empty (Group * group)
{
	if (group)
	{
		/* unload group (if it's loaded) */
		if (articlelist_get_current_group() == group)
			articlelist_clear ();

		/* clear out the article headers */
		ahdb_delete_db_file (group);

		/* wipe out all records */
		group_set_article_qty (group, 0);
		group_set_article_read_qty (group, 0);

		/* flush the flags */
		if (group->flags & GROUP_SUBSCRIBED)
			group_set_flags (group, GROUP_SUBSCRIBED);
		else
			group_set_flags (group, 0);
	}
}

/* callback for button clicks in group_empty_dialog */
static void
group_empty_cb (GnomeDialog *dialog,
		int button_number,
		gpointer group)
{
	switch (button_number)
	{
		case 0: /* 'yes' */
			real_group_empty (group);
			break;

		case 1: /* 'no' */
			break;

		default: /* cancel */
			break;
	};

	gtk_widget_destroy (GTK_WIDGET(dialog));
}

/**
 * group_empty_dialog:
 * @group: group to be emptied
 *
 * dialog popup asking user if they'd like to remove all the messages
 * for a given group.  called when the user tries to unsubscribe
 * or they choose the menu option to empty the group manually
 **/
void
group_empty_dialog (Group *group)
{
	GString * str = g_string_new (NULL);
	GtkWidget *dialog = NULL;
	GtkWidget *label = NULL;

	/* sanity check */
	g_return_if_fail (group!=NULL);

	pan_lock();

	/* create dialog */
	g_string_sprintf (str, _("Clean group \"%s\""), group->name);
	dialog = gnome_dialog_new (str->str,
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO, NULL);

	/* body */
	g_string_sprintf (str, _("Remove all messages from group: \n%s ?"), group->name);
	label = gtk_label_new (str->str);

	/* listen for clicks */
	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_empty_cb), group);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);
	pan_unlock();

	gui_popup_draw (dialog, Pan.window);

	g_string_free (str, TRUE);
}

/*
 * 1. remove article db file
 * 2. remove group from the {group/folder}list
 * 3. save list
 */
static void
real_group_remove (Group *group)
{
	GString * path = g_string_new (NULL);

	/* clear out the article headers */
	ahdb_delete_db_file (group);
	
	grouplist_remove_row (group);

	if (group->type == TYPE_USER_FOLDER)
	{
		pan_db db;

		/* remove folder from folderlist */
		g_string_sprintf (path, "/%s/folderlist.db", data_dir);
		db = pan_db_open (path->str);
		pan_db_erase (db, group->name);
		pan_db_close (db);
		db = NULL;

		/* remove the folder file */
		g_string_sprintf (path, "/%s/folders/%s.db", data_dir, group->name);
		remove (path->str);

		/* remove from current Pan session */
		g_ptr_array_remove (folders, group);
	}
	else if (group->type == TYPE_GROUP)
	{
		pan_db db;
		g_string_sprintf (
			path, "/%s/%s_grouplist.db", data_dir, group->server->name);
		db = pan_db_open (path->str);
		g_ptr_array_remove (group->server->groups, group);
		pan_db_erase (db, group->name);
		pan_db_close (db);
	}

	g_string_free (path, TRUE);
}

/* callback for button clicks in group_remove_dialog */
static void
group_remove_cb (GnomeDialog *dialog,
		 int button_number,
		 gpointer group)
{
	switch (button_number)
	{
		case 0: /* 'yes' */
			real_group_remove (group);
			break;

		case 1: /* 'no' */
			break;

		default: /* cancel */
			break;
	};

	gtk_widget_destroy (GTK_WIDGET(dialog));
}


void
group_remove_dialog (Group *group)
{
	GString * tmp = g_string_new (NULL);
	GtkWidget *dialog = NULL;
	GtkWidget *label = NULL;

	/* sanity check */
	g_return_if_fail (group!=NULL);
	g_return_if_fail (group->name != NULL);

	/* create dialog */
	g_string_sprintf (tmp, _("Remove group \"%s\""), group->name);
	dialog = gnome_dialog_new (tmp->str,
				   GNOME_STOCK_BUTTON_YES,
				   GNOME_STOCK_BUTTON_NO, NULL);

	/* body */
	g_string_sprintf (tmp, _("Remove group: \n%s ?"), group->name);
	label = gtk_label_new (tmp->str);

	/* listen for clicks */
	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (group_remove_cb), group);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), label);

	gui_popup_draw (dialog, Pan.window);
}

void
group_download_all_articles (Group * group)
{
	GPtrArray * articles;
	PanObject * o;
	gpointer article_db;

	g_return_if_fail (group != NULL);

	article_db = ahdb_ref (group);
	articles = ahdb_get_all (article_db, NULL, NULL);
	o = task_bodies_new (group, articles);
	ahdb_unref (article_db);

	queue_add (TASK(o));
}

/**
***
**/

static int
group_catchup_task_run (TaskFuncPtr * tfp, gpointer user_data)
{
	gulong qty=0, low=0, high=0;

	Group * group = GROUP (user_data);
	if (nntp_get_group_info (STATUS_ITEM(tfp),
	                         TASK(tfp)->sock,
	                         group->name,
	                         &qty, &low, &high,
	                         &TASK(tfp)->hint_abort))
		return -1;

	group_set_article_qty (group, qty);
	group_set_article_range (group, low, high);
	newsrc_mark_all (group->newsrc, TRUE);
	group_set_article_read_qty (group, qty);
	return 0;
}

static char*
group_catchup_task_describe (const StatusItem * status)
{
	const TaskFuncPtr * ptr = TASK_FUNC_PTR (status);

	return g_strdup_printf (_("Catching up group `%s'"),
	                        GROUP(ptr->user_data)->name);
}

void
group_catchup (Group * group)
{
	if (articlelist_get_current_group() == group)
	{
		articlelist_all_mark_read ();
	}
	else
	{
		queue_add (TASK(task_func_ptr_new (group->server,
						   group_catchup_task_describe,
						   group_catchup_task_run,
						   group,
						   NULL)));
	}
}

/**
***
**/

static int
group_update_count_info_task_run (TaskFuncPtr * task, gpointer user_data)
{
	GPtrArray * groups = (GPtrArray*) user_data;
	StatusItem * status = STATUS_ITEM(task);
	int retval = 0;
	guint i;

	/* for each group, get the numbers... */
	status_item_emit_init_steps (status, groups->len);
	for (i=0; i!=groups->len; ++i)
	{
		gulong qty=0, low=0, high=0;
		Group * group = GROUP(g_ptr_array_index(groups,i));

		/* let the status item know what we're doing */
		status_item_emit_next_step (status);
		if (!(i % 10))
			status_item_emit_status (status,
				_("Updated counts for %u groups out of %u"),
				i, groups->len);

		/* get the numbers */
		retval = nntp_get_group_info (
			STATUS_ITEM(task),
			TASK(task)->sock,
			group->name,
			&qty, &low, &high,
			&TASK(task)->hint_abort);
		if (retval)
			break;

		/* update the group object */
		group_set_article_qty (group, qty);
		group_set_article_read_qty (
			group,
			group->article_read_qty - (low - group->article_low));
	}

	return retval;
}

static char*
group_update_count_info_task_describe (const StatusItem * status)
{
	return g_strdup_printf (
		_("Updating article counts for %u groups from server `%s'"),
		((GPtrArray*)(TASK_FUNC_PTR(status)->user_data))->len,
		TASK(status)->server->name);
}

static void
group_update_count_free_data (gpointer a)
{
	/* free the gptrarray passed into group_update_count_info */
	g_ptr_array_free ((GPtrArray*)a, TRUE);
}

void
group_update_count_info (GPtrArray * groups)
{
	Server * server;

	g_return_if_fail (groups != NULL);
	g_return_if_fail (groups->len != 0);

	/* the task owns & will free 'groups' now... */
	server = GROUP(g_ptr_array_index(groups,0))->server;
	queue_add (TASK(task_func_ptr_new(server,
	                                  group_update_count_info_task_describe,
	                                  group_update_count_info_task_run,
	                                  groups,
					  group_update_count_free_data)));
}


/**
***
**/

void
group_set_subscribed (Group * group, gboolean subscribed)
{
	g_return_if_fail (group != NULL);

	if (subscribed)
	{
		group_set_dirty (group);
		group_set_flags (group, group->flags|GROUP_SUBSCRIBED);
	}
	else
	{
		group_set_dirty (group);
		group_set_flags (group, group->flags&~GROUP_SUBSCRIBED);

		if (group->article_qty != 0)
			group_empty_dialog (group);
	}

	grouplist_update_row (group);
}

gboolean 
group_is_subscribed (const Group * group)
{
	g_return_val_if_fail (group!=NULL, FALSE);

	return (group->flags & GROUP_SUBSCRIBED) ? 1 : 0;
}

/**
***
**/

void
group_set_new (Group * group, gboolean is_new)
{
	g_return_if_fail (group != NULL);

	group_set_dirty (group);

	if (is_new)
		group_set_flags (group, group->flags|GROUP_NEW);
	else
		group_set_flags (group, group->flags&~GROUP_NEW);

	grouplist_update_row (group);
}

gboolean 
group_is_new (const Group * group)
{
	g_return_val_if_fail (group!=NULL, FALSE);
	return (group->flags & GROUP_NEW) ? 1 : 0;
}

